/*-------------------------------------------------------------------------------------------------\
| Solarboy Solar Bike Controller
|
| File:					bike_ini.C		Initialization Code
|
| Global Routines:	Initialize()
|
| Local Routines:		
|
\-------------------------------------------------------------------------------------------------*/

#include	<pic.h>
#include "bike.h"

//-------------------------------------------------------------------------------------------------
//  Define Locally Global Variables (variables accessible by functions in this file only)
//-------------------------------------------------------------------------------------------------

	// None

//-------------------------------------------------------------------------------------------------	
//  Local #defines
//-------------------------------------------------------------------------------------------------

	// None

//-------------------------------------------------------------------------------------------------	
//  Local Function Prototypes
//-------------------------------------------------------------------------------------------------

	// None

/*-------------------------------------------------------------------------------------------------\
| Function:   Initialize
|
| Purpose:    Initialize the Ports, Peripherals and variables for the RA3 Logic Module
|
| Discussion: All sorts of ports and peripherals need to be initialized and turned on after
|				  a reset (whether it was a MCLR or power up reset).
|             
| Expects:    To be called after any type of reset.
|             Interrupts to be disabled.
|					
|   Date   | Notes
|----------|---------------------------------------------------------------------------------------
| 10/28/01 | Borrow from previous ADG code.
\--------------------------------------------------------------------------------------------------*/

void Initialize (void)
{

	int	i;
	
	// First we need to bring all of the PORTs into their correct state. 

	// Note that we set the PORT registers FIRST, and then the TRIS registers. In this order,
	// we can be sure that the output pins come "up" out of tri-state in the right state.

	// ------
	
	PORTA = 0b00000000; // Note: ADCON1 also affects PORTA and is set below in the ADC section

	TRISA = 0b00111111;
			 // ||||||||
			 // |||||||+----------- In  <- RA0/AN0 = AN0
		    // ||||||+------------ In  <- RA1/AN1 = AN1
		    // |||||+------------- In  <- RA2/AN2 = AN2
		    // ||||+-------------- In  <- RA3/AN3 = AN3
		    // |||+--------------- In  <- RA4     = T0CKI = Battery Amps Pulse Accumulator
			 // ||+---------------- In  <- RA5/AN4 = AN4
		    // |+----------------- <Unimplimented on PIC>
	       // +------------------ <Unimplimented on PIC>

	PORTA = 0b00000000;
	
	// ------

	RBPU = 0;		// Turn on Port B weak (~60kohm) pull-up resistors	
	TRISB = 0xFF;	// PortB all inputs (should already be set that way by reset default)

	// ------

	PORTC = 0b00000000;	// Turn off all outputs

	TRISC = 0b10000001;
			 // ||||||||
			 // |||||||+----------- In  -> RC0 = T1CKI = Motor Amps Pulse Accumulator
		    // ||||||+------------ Out -> RC1 = Spare digital
		    // |||||+------------- Out -> RC2 = Spare digital
		    // ||||+-------------- Out -> RC3 = Spare digital
		    // |||+--------------- Out -> RC4 = Spare digital
			 // ||+---------------- Out -> RC5 = Spare digital
		    // |+----------------- Out -> RC6 = UART Transmit - Linx 900MHz HP-II
	       // +------------------ In  <- RC7 = UART Receive  - GPS

	PORTC = 0b00000000;	// Turn off all outputs

	// ------

	PORTD = 0b00000000;

	TRISD = 0b00000000;
			 // ||||||||
			 // |||||||+----------- Out -> RD0 = LCD D0
  		 	 // ||||||+------------ Out -> RD1 = LCD D1
  		 	 // |||||+------------- Out -> RD2 = LCD D2
  			 // ||||+-------------- Out -> RD3 = LCD D3
   		 // |||+--------------- Out -> RD4 = E
			 // ||+---------------- Out -> RD5 = R/W
   		 // |+----------------- Out -> RD6 = RS
   		 // +------------------ Out -> RD7 = Unused
   		 
	PORTD = 0b00000000;

	// ------

	PORTE = 0b00000000;

	TRISE = 0b00000111;
			 // ||||||||
			 // |||||||+----------- In  <- RE0 = AN5
		    // ||||||+------------ In  <- RE1 = AN6
		    // |||||+------------- In  <- RE2 = AN7
		    // ||||+-------------- <Unimplimented on PIC>
		    // |||+--------------- PSPMode = 0
			 // ||+---------------- IBOV (Read Only)
		    // |+----------------- OBF  (Read Only)
	       // +------------------ IBF  (Read Only)

	PORTE = 0b00000000;

	// ---------------------------------------------------------------------------
	// Before turning on any peripherals, initialize all relevant data so that
	// the timer won't time out, or the receive buffer won't overflow, etc., as we
	// initialize the data.                                            

	gbPowerUp = 1;			// Cleared in INT/Receive (GPS)

	// Set the sign bits for the battery and motor amphour meter
	
	i = PORTB;					// Read the current bits
	RBIF = 0;					// clear the interrupt flag

	gbPosBattAmps = 0;
	if (RB4 == 1)
		gbPosBattAmps = 1;
	
	gbPosMotorAmps = 0;
	if (RB5 == 1)
		gbPosMotorAmps = 1;
	
	// ---------------------------------------------------------------------------
	// Bring the peripherals online. This means Timer0, Timer1, Timer2, the UART and the 10bit ADC.

	// Initialize the 8bit Timer0 to be a pulse accumulator (bits found in OPTION_REG)

	T0CS = 1;	// Timer0 transitions on RA4
	T0SE = 0;	// Timer0 increments on low to high transition
	PSA = 1;		// Prescaler is assigned to the WDT (don't need it for Timer0)
	TMR0 = 0;	// Clear the timer
	
	// Initiailze the 16bit Timer1 to be a 8bit pulse accumulator using the compare function
	
	T1CON = 0b00000011;
			 // ||||||||
			 // |||||||+----------- TMR1ON = 1 (on)
		    // ||||||+------------ TMR1CS = 1 (external clock)
		    // |||||+------------- T1SYNC = 0 (synchronize to match Timer0; must be sync to use CCP)
		    // ||||+-------------- T1OSCEN = 0
		    // |||+--------------- T1CKPS0 = 00 = 1:1
			 // ||+---------------- T1CKPS1
 		    // |+----------------- <Unimplimented on PIC>
	       // +------------------ <Unimplimented on PIC>

	CCPR1L = 0xFF; //	Interrupt and clear Timer1 when Timer 1 = 0x0100
	CCPR1H = 0x00;

	CCP1CON = 0b00001011;
			 // 	||||||||
			 // 	|||||||+----------- CCP1M0 = CCP Mode 0:3 = 1011 = Compare mode, clear TMR1 on match 
		    // 	||||||+------------ CCP1M1
		    // 	|||||+------------- CCP1M2
		    // 	||||+-------------- CCP1M3
		    // 	|||+--------------- CCP1Y (PWM MSB)
			 // 	||+---------------- CCP1X (PWM MSB)
 		    // 	|+----------------- <Unimplimented on PIC>
	       // 	+------------------ <Unimplimented on PIC>

	TMR1L = 0;		// Clear the timer
	TMR1H = 0;		//
	
	// Initialize Timer2  (20MHz/4 / 1:4 / 250 / 1:10 = 500Hz)
	
	T2CON = 0b01001101;
			 // ||||||||
			 // |||||||+----------- T2CKPS0 = 01  (Prescaler is to 1:4)
		    // ||||||+------------ T2CKPS1
		    // |||||+------------- TMR2ON  = 1   (Timer2 is on)
		    // ||||+-------------- TOUTPS0 = 1001 (1:10 Postscaler)
		    // |||+--------------- TOUTPS1
			 // ||+---------------- TOUTPS2
		    // |+----------------- TOUTPS3
	       // +------------------ <Unimplimented on PIC>
	
	PR2 = 249;		// Set the period register for Timer to 250
	
	// Initialize the ADC and PORTA for the ADC channels.
	
	ADCON0 = 0b10000001;
			//   ||||||||	
			//   |||||||+----------- ADON  = 1   (Turn the ADC on)
			//   ||||||+------------ <Unimplimented on PIC>
		  	//   |||||+------------- GO    = 0   (Don't start a conversion)
			//   ||||+-------------- CHS0  = 000 (Start with Channel 0)
			//   |||+--------------- CHS1
			//   ||+---------------- CHS2
			//   |+----------------- ADCS0 = 10  (Fosc/32)
			//   +------------------ ADCS1

	ADCON1 =	0b10000000;
			//   ||||||||
			//	  |||||||+----------- PCFG0 = 0000 (RA0 - RA3, RA5, RE0-2 are AN0 - AN7)
			//	  ||||||+------------ PCFG1
			//	  |||||+------------- PCFG2
			//	  ||||+-------------- PCFG3
			//	  |||+--------------- <Unimplimented on PIC>
			//	  ||+---------------- <Unimplimented on PIC>
			//	  |+----------------- <Unimplimented on PIC>
			//	  +------------------ ADFM = 1     (Right justified: ADRESH = ADC9:8, ADRESL = ADC7:0)

	// Initialize the UART for receive and transmission

	SPBRG = 129;	// Set the bps rate to 2400bps (actually 2404bps = +0.17% error) given BRGH = 0
	
	RCSTA = 0b10010000;
			//  ||||||||	
			//  |||||||+----------- RX9D
			//  ||||||+------------ OERR
			//  |||||+------------- FERR
			//  ||||+-------------- ADDEN = 0
			//  |||+--------------- CREN  = 1
			//  ||+---------------- SREN
			//  |+----------------- RX9   = 0
			//  +------------------ SPEN  = 1

	TXSTA = 0b00100000;
			//  ||||||||	
			//  |||||||+----------- TX9D  (unused: 9th bit) 
			//  ||||||+------------ TRMT
			//  |||||+------------- BRGH = 0 (low speed)
			//  ||||+-------------- Unimplemented
			//  |||+--------------- SYNC = 0 (asynchronous)
			//  ||+---------------- TXEN = 1 (enable transmission)
			//  |+----------------- TX9D = 0 (8 data bits)
			//  +------------------ CSRC

	// ---------------------------------------------------------------------------

	// Clear any pending interrupts or error flags

	// i = PORTB;	// DO NOT CLEAR PORTB INTERRUPT ON CHANGE! LET THE ISR DO IT!
	// RBIF = 0;	// ... since we don't want to miss a change since the init code above.

	T0IF = 0;				// Clear any pending timer interrupt
	CCP1IF = 0;
	TMR2IF = 0;
	
	i = RCREG;				// Clear any incoming data and overrun errors in the UART receive buffer
	CREN = 1;				

	// Turn on all of the interrupt enables and finally the global enables.

	RBIE = 1;				// Enable PORTB Interrupt on change (IOC)
	
	RCIE = 1;				// Enable UART interrupt on receive buffer full (GPS)
	TXIE = 1;				// Enable UART interrupt on transmit buffer empty (Linx)

	CCP1IE = 1;				// Enable Timer1 Compare match interrupts
	T0IE = 1;				// Enable Timer0 overflow interrupts
	TMR2IE = 1;				// Enable Timer2 overflow interrupts
	
	PEIE = 1;				// Enable peripheral interrupts
	GIE = 1;					// Enable global interrupts
	
}

