/*-------------------------------------------------------------------------------------------------\
| Solarboy Solar Bike Controller
|
| File:					BIKE_INT.C		Interrupt Routines
|
| Global Routines:	ISR()
|
| Local Routines:		Transmit()
|							Receiver()
|							PortBIOC()
|							Timer0Int()
|							CCP1Int()
|							Timer2Int()
|
| IMPORTANT NOTE ON THE STRUCTURE OF THIS MODULE:
| -----------------------------------------------
| PICC will automatically save the context of all functions that an "interrupt"-labeled routine
| (ISR below) calls. However, the functions called must be ABOVE THE INTERRUPT-LABELED FUNCTION
| in the same code module (i.e., the same file). Thus, this module must RETAIN THE ORDER OF THE
| FUNCTIONS with the interrupt function being last in the module. For more information, see the
| Hi-Tech PIC C Manual.
\-------------------------------------------------------------------------------------------------*/

#include	<pic.h>
#include "bike.h"


//-------------------------------------------------------------------------------------------------
//  Define Locally Global Variables (variables accessible by functions in this file only)
//-------------------------------------------------------------------------------------------------

	// None

//-------------------------------------------------------------------------------------------------
//  Local #defines
//-------------------------------------------------------------------------------------------------

	// None

//-------------------------------------------------------------------------------------------------	
//  Local Function Prototypes
//-------------------------------------------------------------------------------------------------

	void	Receive		(void);
	void	Transmit		(void);
	void	Receiver		(void);
	void	PortBIOC		(void);
	void	Timer0Int	(void);
	void	CCP1Int		(void);
	void	Timer2Int	(void);



/*-------------------------------------------------------------------------------------------------\
| Function:  	PortBIOC
|
| Purpose:   	Discover which bit on PORTB RB4:7 flipped
|
| Discussion:	NOTE: ICD CONFLICT! AUGH!
|
| Expects:		To be called on a change of state on RB4:7
|					Interrupts to be disabled.
|					
|   Date   | Notes
|----------|---------------------------------------------------------------------------------------
| 10/28/01 | Stolen from other ADG code
\--------------------------------------------------------------------------------------------------*/
   
void PortBIOC (void)
{
	static	char	CurrPortB;
	static	char	IocTemp;
	static	char	prevPortB;
	
	CurrPortB = PORTB;
	RBIF = 0;

	IocTemp = prevPortB ^ CurrPortB;		// XOR the value to see which bits have flipped.

	//------------------------------------------------------------------------------------------
	// Change in RB4 = Sign on the battery amps has changed
	
	if (IocTemp & 0b00010000)
	{
		if (CurrPortB & 0b00010000)
		{
			// RB4 changed and it's now high, so we've switched from low to high (- to +)
			ampHours -= TMR0;
			TMR0 = 0x00;
			gbPosBattAmps = 1;
		}
		else
		{
			// RB4 changed and it's now low, so we've switched from high to low (+ to -)
			ampHours += TMR0;
			TMR0 = 0x00;
			gbPosBattAmps = 0;
		}
	}

	//------------------------------------------------------------------------------------------
	// Change in RB5 = Sign on the motor amps has changed

	if (IocTemp & 0b00100000)
	{
		if (CurrPortB & 0b00100000)
		{
			// RB5 changed and it's now high, so we've switched from low to high (- to +)
			gbPosMotorAmps = 1;
		}
		else
		{
			// RB5 changed and it's now low, so we've switched from high to low (+ to -)
			gbPosMotorAmps = 0;
		}
	}

	//------------------------------------------------------------------------------------------
	// Change in RB6 = Speed reed relay has changed

	if (IocTemp & 0b01000000)
	{
		if (CurrPortB & 0b01000000)
		{
			// RB6 has gone from LOW TO HIGH; this means we've gotten a pulse from the speed sensor.
			// Flag the Timer2 interrupt that we've had a transition so it can reset the counter
			// and calculate the speed. Timer2 is expected to clear the bit.

			gbSpeedSensor = 1;
		}
	}
		
	//------------------------------------------------------------------------------------------
	// Change in  RB7 = Cadence reed relay has changed
		
	if (IocTemp & 0b10000000)
	{
		if (!(CurrPortB & 0b10000000))
		{
			// RB7 has gone from HIGH TO LOW; this means we've gotten a pulse from the cadence sensor.
			// Flag the Timer2 interrupt that we've had a transition so it can reset the counter
			// and calculate the cadence. Timer2 is expected to clear the bit.

			gbCadenceSensor = 1;
		}
	}

	prevPortB = CurrPortB;
}



/*-------------------------------------------------------------------------------------------------\
| Function:  	Receive
|
| Purpose:   	Grabs the byte in the UART receive buffer and processes it.
|
| Discussion:	Bytes coming in the UART are from the GPS, usually Rockwell binary message 1000.
|					We only want some of the data, so first we make sure it's message 1000 and then
|					we grab only the bytes we want and store that in the appropriate queue.
|
| Expects:		To be called on the UART Receiver buffer full interrupt.
|					Interrupts to be disabled.
|
|   Date   | Notes
|----------|---------------------------------------------------------------------------------------
| 10/28/01 | Stolen from other ADG code.
| 11/10/00 | Moved to C.
\--------------------------------------------------------------------------------------------------*/
   
void Receive (void)
{
	static char				rxTemp;
	static char				rxTemp2;
	static signed int		rxTempInt;
	static unsigned int	rxTempUInt;
	
	static char		rxByteNum;
	static char		num2Skip;
	
	static char		pktByteNum;
	const	char		gpsByteNum[] = {36,42,44,46,52,53,54,55,56,57,58,59,60,61};


	// First we clear the interrupt by reading the UART Receive register

	rxTemp = RCREG;		

	// Should we be skipping the next few bytes? If skipBytes > 0, then we should
	// be skipping num2Skip number of bytes. Otherwise, we jump into our giant
	// switch statement to figure out what to do with the byte.

	if (num2Skip > 0)
	{
		++rxByteNum;
		--num2Skip;
	}

	else
	{
		if (rxByteNum < 25)
		{
			switch (rxByteNum)
			{
				case 0:									// Word 1: 0xFF (LSB header)
		
					if (rxTemp == 0xFF)
						++rxByteNum;
					break;

				case 1:									// Word 1: 0x81 (MSB header)
		
					if (rxTemp == 0x81)
						++rxByteNum;
					else
					{
						rxByteNum = 0;
						if (rxTemp == 0xFF)
							++rxByteNum;
					}		
					break;
		
				case 2:									// Word 2: 0xE8 (LSB 1000)

					if (rxTemp == 0xE8)
						++rxByteNum;
					else
					{
						rxByteNum = 0;
						if (rxTemp == 0xFF)
							++rxByteNum;
					}		
					break;

				case 3:									// Word 2: 0x03 (MSB 1000)

					if (rxTemp == 0x03)
					{
						++rxByteNum;
						gbGPSOk = 1;					// We've received the header, so flag the GPS is ok.
						num2Skip = 14;					// (one second flags are at the end)

						//------------------------
						#ifdef	GOTGPSTOD7
						RD7 = 1;
						#endif
						//------------------------
					}
					
					else
					{
						rxByteNum = 0;
						if (rxTemp == 0xFF)
							++rxByteNum;
					}		
					break;

				case 18:									// Word 10.0 = Nav soln validity

					rxTemp2 = 0;
					if (rxTemp & 0b00000100)		// 	Move 10.2 (not enough sats) to bit 7
						rxTemp2 |= 0b10000000;
						
					if (rxTemp & 0b00011000)		// 	Move (10.3 | 10.4) to bit 6 (exceeded EHPE/EHVE)
						rxTemp2 |= 0b01000000;

					++rxByteNum;
					num2Skip = 1;
					break;
				
				case 20:									// Word 11.0 = Nav soln type 

					rxTemp &= 0b00000011;
					rxTemp2 |= (rxTemp <<4);		// 	Move 11.1 and 11.0 (solution type pop & alt) to bit 5 & 4
					++rxByteNum;
					num2Skip = 1;
					break;

				case 22:									// Word 12.0 = Num. Measurements

					rxTemp &= 0b00001111;			//		Add the # of sats to the lower nibble
					rxTemp |= rxTemp2;
					IsrWrite2Packet (PKT_SATLOCK, rxTemp);
					++rxByteNum;
					num2Skip = 13;
					break;

				default:

					++rxByteNum;
					break;
			}
		}
		
		else if (rxByteNum < 62)
		{
			// Process everything that can be just scooted over to the packet from the GPS

			if ( rxByteNum == gpsByteNum[pktByteNum] )
			{
				IsrWrite2Packet (pktByteNum+1, rxTemp);
				++pktByteNum;
			}
			++rxByteNum;
		}
		
		else if (rxByteNum < 92)
		{
			switch (rxByteNum)
			{
				case 64:									// Word 33.0 = Geoidal separation

					rxTemp2 = rxTemp;
					++rxByteNum;
					break;

				case 65:									// Word 33.1 = Geoidal separation

					rxTempInt = (rxTemp<<8) + rxTemp2;			// Make an int of the the two bytes
					rxTempInt = rxTempInt >> 1;						// divide by 2
					IsrWrite2Packet(PKT_GEOSEP, (char)rxTempInt);	// Use the lower byte only
					++rxByteNum;
					break;


				case 66:									// Word 34.0 = speed

					IsrWrite2Packet(PKT_GPSSPEED0, rxTemp);
					++rxByteNum;
					break;

				case 67:									// Word 34.1 = speed

					IsrWrite2Packet(PKT_GPSSPEED1, rxTemp);
					++rxByteNum;
					num2Skip = 2;
					break;

				case 70:									// Word 36.0 = True Course

					rxTemp2 = rxTemp;
					++rxByteNum;
					break;

				case 71:									// Word 36.1 = True Course

					rxTempUInt = (rxTemp<<8) + rxTemp2;					// Make an int of the the two bytes
					rxTempUInt = rxTempUInt >> 5;							// divide by 2
					IsrWrite2Packet(PKT_HEADING, (char)rxTempUInt);	// Use the lower byte only
					++rxByteNum;
					num2Skip = 2;
					break;

				case 74:									// Word 38.0 = Climb rate

					rxTemp2 = rxTemp;
					++rxByteNum;
					break;

				case 75:									// Word 38.1 = Climb rate

					rxTempInt = (rxTemp<<8) + rxTemp2;			// Make a signed int of the the two bytes
					rxTempInt = rxTempInt >> 1;					// divide by 2
					IsrWrite2Packet(PKT_CLIMB, rxTempInt);		// Use the lower byte only
					++rxByteNum;
					num2Skip = 3;
					break;

				case 79:									// Word 40.0 = EHPE

					IsrWrite2Packet(PKT_EHPE, rxTemp);
					++rxByteNum;
					num2Skip = 10;
					break;

				case 90:									// Word 46.0 = EVPE

					rxTemp2 = rxTemp;
					++rxByteNum;
					break;
					
				case 91:									// Word 46.1 = EVPE

					if ((rxTemp & 0b10000000) || (rxTemp & 0b01000000))
						IsrWrite2Packet(PKT_EHVE, 0xFF);				// If the MSB is > 14bits then saturate the reading
					else
					{
						rxTempUInt = (rxTemp<<8) + rxTemp2;			// Make an UNSIGNED int of the the two bytes
						rxTempUInt = rxTempUInt >> 2;					// divide by 4	
						IsrWrite2Packet(PKT_EHVE, rxTempUInt);		// Use the lower byte only
					}
					++rxByteNum;
					break;

				default:
					++rxByteNum;
					break;	
			}
		}

		else
		{
			// This is close to the end of message 1000. Flag that we've received the message and can
			// begin processing everything else... we would like to avoid jitter here, and based on
			// the above code there shouldn't be at this point - maybe about 20 Tcyc of latency (4us).
			// Note that we HAVE to be done with the GPS packet before we start the main line routine,
			// because at the end of the main routine it'll update the gPacketNum to the next packet.
			// If this gPacketNum switches before either routine is finished, it'll screw everything up.

			if (gbPowerUp)
			{
				gbPowerUp = 0;				// Skip the first GPS message
			}
			else
			{
				if (gbStartMain)			// Check for main not being done yet
					IsrFault(2);

				gbStartMain = 1;			// Start the packet stuffing process

				// Now we've flagged everyone, so now reset the GPS counters.
			}
			
			rxByteNum = 0;				// Reset GPS message search
			pktByteNum = 0;			// Reset GPS move function'

			//------------------------
			#ifdef	GOTGPSTOD7
			RD7 = 0;
			#endif
			//------------------------
		}
	}	
}



/*-------------------------------------------------------------------------------------------------\
| Function:  	Transmit
|
| Purpose:   	Stuff the UART Transmit buffer with data from one of the frame Queues or empty data.
|
| Discussion:	Every second, as ticked off by the GPS or the onboard Timer2 one second counter,
|					triggers this routine to send off all data packets except the one currently being
|					built - e.g., the last three data packets are sent. gPacketNum holds the packet
|					currently being assembled. So we decrement and send the previous ones - up to the
|					number of "allowed packet transmissions" - at 2400bps and 45 char/packet, that's
|					5 packets each cycle.
|
| Expects:		To be called on the UART TX buffer empty interrupt.
|					Interrupts to be disabled.
|					
|   Date   | Notes
|----------|---------------------------------------------------------------------------------------
| 10/28/01 | Stolen from other ADG code
\--------------------------------------------------------------------------------------------------*/

void Transmit (void)
{

//	static	char 		txPacketNum;
	static	char		txState;
	static	char		txByteNum;
	static	char		txPacketCnt;
	
	if (!gbStartTx)
	{
		TXREG = 0x55;			// We've already sent the packets. Send out 0x55's as filler for the Linx.
	}

	else
	{
		if (txState == 0)
		{
			TXREG = PKT_HEADER0;
			++txState;
		}

		else if (txState == 1)
		{
			// Send the first byte of the header when we're looping through packets
			TXREG = PKT_HEADER1;
			++txState;
		}

		else if (txState == 2)
		{
			TXREG = PKT_HEADER2;
			++txState;
		}

		else
		{

			if (gTxPacketNum == 3)						// Depending on the LOCAL packet #,
				TXREG = gPacket3[txByteNum];			// send the next byte. CAN'T use
			else if (gTxPacketNum == 2)					// ReadPacket function.
				TXREG = gPacket2[txByteNum];
			else if (gTxPacketNum == 1)
				TXREG = gPacket1[txByteNum];
			else
				TXREG = gPacket0[txByteNum];

			++txByteNum;
			
			if (txByteNum >= PACKETLENGTH)
			{
				++txPacketCnt;
				
				if (txPacketCnt >= PACKETS_TX_PER_SEC)
				{
					txPacketCnt = 0;		// We've sent all the packets we want.
					gbStartTx = 0;			// Reset the state machine and let the GPS routine know we're done.
					txState = 0;			// Zero to start the next second correctly.
				}
				
				else
				{
					txState = 0;			// Set the state to send the next packet this second.
	
					if (gTxPacketNum == 0)
						gTxPacketNum = (NUMPACKETS-1);
					else
						--gTxPacketNum;
				}
				
				txByteNum = 0;
			}
		}		
	}
}



/*-------------------------------------------------------------------------------------------------\
| Function:  	Timer0Int
|
| Purpose:   	Handle the interrupt on Timer0 overflow - Battery amps and amp hours.
|
| Discussion:	We use the Timer0 as an 8 bit pulse accumulator.
|
| Expects:		To be called on Timer0 overflow.
|					Interrupts to be disabled.
|					
|   Date   | Notes
|----------|---------------------------------------------------------------------------------------
| 10/28/01 | Stolen from other ADG code
\--------------------------------------------------------------------------------------------------*/
   
void Timer0Int (void)
{
	T0IF = 0;		// Clear the interrupt

	if (gbPosBattAmps)
	{
		ampHours += 0x0100;	// Add 1 to the 2nd byte
		++battAmps;
	}
	
	else
	{
		ampHours -= 0x0100;	// Subtract 1 to 2nd byte
		--battAmps;
	}
}




/*-------------------------------------------------------------------------------------------------\
| Function:  	CCP1Int
|
| Purpose:   	Handle the interrupt on CCP1 Match.
|
| Discussion:	We brewed our own 8bit pulse accumulator by using the 16bit Timer1 in CCP1 mode.
|					Everytime the pulse accumulator = 0x100, it resets the count and generates an
|					input.
|
| Expects:		To be called on the CCP1 compare matches interrupt.
|					Interrupts to be disabled.
|					
|   Date   | Notes
|----------|---------------------------------------------------------------------------------------
| 10/28/01 | Stolen from other ADG code
\--------------------------------------------------------------------------------------------------*/
   
void CCP1Int (void)
{
	
	CCP1IF = 0;		// Clear the interrupt

	if (gbPosMotorAmps)
	{
		++motorAmps;
	}
	
	else
	{
		--motorAmps;
	}

}




/*-------------------------------------------------------------------------------------------------\
| Function:  	Timer2Int
|
| Purpose:   	Handle the interrupt on Timer2 overflow & increment a free running 500Hz / 2ms clock.
|
| Discussion:	Timer2Int runs a free running clock for the Speed/Cadence sensors. Every rising edge
|					PortB interrupt on change sets a flag for this routine which tells it to store the
|					the count.
|
| Expects:		To be called on Timer2 overflow.
|					Interrupts to be disabled.
|					
|   Date   | Notes
|----------|---------------------------------------------------------------------------------------
| 10/29/01 | Stolen from other ADG code
\--------------------------------------------------------------------------------------------------*/

void Timer2Int (void)
{

	static unsigned int	cadenceTick;			// 1 tick = 1/500Hz = 2ms
	static unsigned int	speedTick;				// 1 tick = 1/500Hz = 2ms
//	static char				t2Temp;

	static unsigned int	millisecs;
	static char				seconds;
	static char				minutes;
	static char				hours;

	static unsigned int	cntGPSOff;
	static bit				bGPSOff;
	

	TMR2IF = 0;		// Clear the interrupt

	//-----------------------------------------------------------------------
	// Process the speed

	++speedTick;

	// If the count goes over 4096 then it's been over 8.192s since we've had a
	// sensor reading, which means we're stopped. Since a valid reading could be
	// 0xFFF, we'll send 0x000 as the "stopped" value.
	
	if (speedTick > 4096)
	{

		speed0 = 0x00;
		speedCadence1 &= 0x0F;		
		speedTick = 0;
	}

	// Now we see if there's been a rising edge speed sensor transition. If so,
	// then we load the packet with the time since last pulse (1/speed) and
	// clear the count and sensor flag
	
	if (gbSpeedSensor)
	{
		// To debounce the speed sensor, we require minimum time between
		// sensor ticks to be at least 22 ticks = 44.6ms = 22.41Hz = 1344rpm
		
		if (speedTick < 22)
			gbSpeedSensor = 0;

		else
		{
			// We have a valid sensor signal, so we put the 12bit count in the current packet

			speed0 = speedTick;										// write the low byte
			speedCadence1 &= 0x0F;									// clear the high nibble
			speedCadence1 |= ((speedTick>>4) & 0xF0);			// write the high nibble
		
			// Now we reset the speed count for the next pulse, and clear the
			// sensor bit
		
			speedTick = 0;
			gbSpeedSensor = 0;
		}
	}

	//-----------------------------------------------------------------------
	// Process the cadence

	++cadenceTick;

	// If the count goes over 4096 then it's been over 8.192s since we've had a
	// sensor reading, which means we're stopped. Since a valid reading could be
	// 0xFFF, we'll send 0x000 as the "stopped" value.
	
	if (cadenceTick > 4096)
	{

		cadence0 = 0x00;
		speedCadence1 &= 0xF0;
		cadenceTick = 0;
	}

	// Now we see if there's been a rising edge speed sensor transition. If so,
	// then we load the packet with the time since last pulse (1/speed) and
	// clear the count and sensor flag
	
	if (gbCadenceSensor)
	{
		// To debounce the speed sensor, we require minimum time between
		// cadence sensor ticks to be at least 150 ticks = 300ms = 3.333Hz = 200rpm
		
		if (cadenceTick < 150)
			gbCadenceSensor = 0;

		else
		{

			// We got a sensor signal, so we put the 12bit count in the current packet

			cadence0 = cadenceTick;									// write the low byte
			speedCadence1 &= 0xF0;									// clear the high nibble
			speedCadence1 |= ((cadenceTick>>8) & 0x0F);		// write the high nibble
	
			// Now we reset the speed count for the nex pulse, and clear the
			// sensor bit
		
			cadenceTick = 0;
			gbCadenceSensor = 0;
		}
	}

	
	//-----------------------------------------------------------------------
	// Create a 1 second clock if the GPS is not working.

	// If the GPS fails, we still want everything to work properly. So we do a "watchdog"
	// task here on the GPS, and if the GPS fails we begin generating the 1s signal here.
	// We wait 2 seconds for the GPS to clear the flag before we take over.

	if (gbGPSOk)
	{
		gbGPSOk = 0;			// GPS is ok, Clear the watchdog signal
		bGPSOff = 0;			// In case the GPS just came back on line, clear the off flag
		cntGPSOff = 0;			//
	}

	else
	{
		if (bGPSOff)
		{

			++millisecs;
			if (millisecs >= 500)		// Wait 500 ticks (1,000ms) to increment a second
			{
				millisecs = 0;
				++seconds;
				if (seconds >= 60)
				{
					seconds = 0;
					++minutes;
					if (minutes >= 60)
					{
						minutes = 0;
						++hours;
					}
				}

				IsrWrite2Packet(PKT_SATLOCK,0xFF);		// Again, bad GPS flag
				IsrWrite2Packet(PKT_DAY, 0);				// GPS is off, flag that by writing 0 to the days
				IsrWrite2Packet(PKT_HOURS, hours);
				IsrWrite2Packet(PKT_MIN, minutes);
				IsrWrite2Packet(PKT_SEC, seconds);
	
				if (gbStartMain)		// Check for main not being done yet
					IsrFault(3);
					
				gbStartMain = 1;		// Start the packet stuffing process
			}
		}
		else
		{
			++cntGPSOff;
		
			if (cntGPSOff >= 1000)		// Wait 1,000 ticks (2000ms) to declare the GPS dead
			{
				bGPSOff = 1;
				cntGPSOff = 0;
			}
		}
	}
}



/*-------------------------------------------------------------------------------------------------\
| Function:  	ISR
|
| Purpose:   	ISR Is the interrupt service routine. It calls the ISR's for each type of interrupt.
|
| Discussion:	There are a million (5) kinds of interrupt in this damn program. The UART receive and
|					transmit, CCP1 and Timer0 for the motor and battery amp meters, and RBIF for the
|					the speed and cadence switches (as well as the sign bits for the amp meters). Note that
|					it can call all of the routines in order if necessary, so the main routine should
|					be pretty latency insensitive (as should the other ISR's!). The more real time the
|					interrup, the higher on the call list it should be.
|
| Expects:		To be called on any kind of PIC16F877 interrupt.
|					For global interupts to be disabled by the interrupt (INTSTA, GIE = 0).
|					Hi-Tech C to save and restore all variables used in and by these int. routines.
|					
|   Date   | Notes
|----------|---------------------------------------------------------------------------------------
| 10/28/01 | Stolen from old ADG code.
\--------------------------------------------------------------------------------------------------*/

void interrupt ISR (void)
{
	// Now we determine who interrupted us and go to that ISR. Note that the order of checking
	// the interrupt flags determines the interrupt priority.

	//---------------------------
	#ifdef	ISRTIMING2PORTD
	RD0 = 1;
	#endif
	//---------------------------

	if (RBIF)			// Check for interrupt on change pins
	{
		//---------------------------
		#ifdef	ISRTIMING2PORTD
		RD1 = 1;
		#endif
		//---------------------------
		PortBIOC();
		//---------------------------
		#ifdef	ISRTIMING2PORTD
		RD1 = 0;
		#endif
		//---------------------------
	}

	if (RCIF)			// Check the UART Receive Buffer Full interrupt
	{
		//---------------------------
		#ifdef	ISRTIMING2PORTD
		RD2 = 1;
		#endif
		//---------------------------
		Receive();
		//---------------------------
		#ifdef	ISRTIMING2PORTD
		RD2 = 0;
		#endif
		//---------------------------
	}

	if (TXIF)			// Check the UART Transmit Buffer Empty interrupt
	{
		//---------------------------
		#ifdef	ISRTIMING2PORTD
		RD3 = 1;
		#endif
		//---------------------------
		Transmit();
		//---------------------------
		#ifdef	ISRTIMING2PORTD
		RD3 = 0;
		#endif
		//---------------------------
	}

	if (T0IF)			// Check for Timer0 overflow interrupt (battery amps)
	{
		//---------------------------
		#ifdef	ISRTIMING2PORTD
		RD4 = 1;
		#endif
		//---------------------------
		Timer0Int();
		//---------------------------
		#ifdef	ISRTIMING2PORTD
		RD4 = 0;
		#endif
		//---------------------------
	}
	
	if (CCP1IF)			// Check the Timer1 CCP compare interrupt (motor amps)
	{
		//---------------------------
		#ifdef	ISRTIMING2PORTD
		RD5 = 1;
		#endif
		//---------------------------
		CCP1Int();
		//---------------------------
		#ifdef	ISRTIMING2PORTD
		RD5 = 0;
		#endif
		//---------------------------
	}
	
	if (TMR2IF)			// Check for Timer2 overflow interrupt (1sec clock)
	{
		//---------------------------
		#ifdef	ISRTIMING2PORTD
		RD6 = 1;
		#endif
		//---------------------------
		Timer2Int();
		//---------------------------
		#ifdef	ISRTIMING2PORTD
		RD6 = 0;
		#endif
		//---------------------------
	}

	//---------------------------
	#ifdef	ISRTIMING2PORTD
	RD0 = 0;
	#endif
	//---------------------------

	// Note: This routine Automatically RETFIEs (which enables interrupts)
}

